@@ -81,6 +81,8 @@ class IrrepsArray:
         self._list = list
 
         if _perform_checks:
+            if not isinstance(self.array, (np.ndarray, jnp.ndarray)):
+                raise ValueError(f"IrrepsArray: Array must be a jax.numpy.ndarray, got {type(self.array)}")
             if self.array.shape[-1] != self.irreps.dim:
                 raise ValueError(
                     f"IrrepsArray: Array shape {self.array.shape} incompatible with irreps {self.irreps}. "
@@ -764,7 +766,7 @@ class IrrepsArray:
         ]
         return IrrepsArray.from_list(self.irreps, new_list, self.shape[:-1], self.dtype)
 
-    def transform_by_angles(self, alpha: float, beta: float, gamma: float, k: int = 0) -> "IrrepsArray":
+    def transform_by_angles(self, alpha: float, beta: float, gamma: float, k: int = 0, inverse: bool = False) -> "IrrepsArray":
         r"""Rotate the data by angles according to the irreps.
 
         Args:
@@ -772,6 +774,7 @@ class IrrepsArray:
             beta (float): second rotation angle around the first axis (in radians)
             gamma (float): first rotation angle around the second axis (in radians)
             k (int): parity operation
+            inverse (bool): if True, apply the inverse rotation
 
         Returns:
             `IrrepsArray`: rotated data
@@ -782,10 +785,12 @@ class IrrepsArray:
             >>> x.transform_by_angles(jnp.pi, 0, 0)
             1x2e [ 0.1 -2.   1.  -1.   1. ]
         """
-        alpha = jnp.asarray(alpha, dtype=self.dtype)
-        beta = jnp.asarray(beta, dtype=self.dtype)
-        gamma = jnp.asarray(gamma, dtype=self.dtype)
+        alpha = alpha if isinstance(alpha, (int, float)) else jnp.asarray(alpha, dtype=self.dtype)
+        beta = beta if isinstance(beta, (int, float)) else jnp.asarray(beta, dtype=self.dtype)
+        gamma = gamma if isinstance(gamma, (int, float)) else jnp.asarray(gamma, dtype=self.dtype)
         D = {ir: ir.D_from_angles(alpha, beta, gamma, k) for ir in {ir for _, ir in self.irreps}}
+        if inverse:
+            D = {ir: jnp.swapaxes(D[ir], -2, -1) for ir in D}
         new_list = [
             jnp.reshape(jnp.einsum("ij,...uj->...ui", D[ir], x), self.shape[:-1] + (mul, ir.dim)) if x is not None else None
             for (mul, ir), x in zip(self.irreps, self.list)
@@ -1141,35 +1146,105 @@ def stack(arrays: List[IrrepsArray], axis=0) -> IrrepsArray:
     )
 
 
-def norm(array: IrrepsArray, *, squared: bool = False) -> IrrepsArray:
+def norm(array: IrrepsArray, *, squared: bool = False, per_irrep: bool = True) -> IrrepsArray:
     """Norm of IrrepsArray.
 
     Args:
         array (IrrepsArray): input array
         squared (bool): if True, return the squared norm
+        per_irrep (bool): if True, return the norm of each irrep individually
 
     Returns:
         IrrepsArray: norm of the input array
 
     Examples:
-        >>> x = e3nn.IrrepsArray("2x0e + 1e + 2e", jnp.arange(10))
+        >>> x = e3nn.IrrepsArray("2x0e + 1e + 2e", jnp.arange(10.0))
         >>> e3nn.norm(x)
         2x0e+1x0e+1x0e [ 0.     1.     5.385 15.969]
+
+        >>> e3nn.norm(x, squared=True)
+        2x0e+1x0e+1x0e [  0.   1.  29. 255.]
+
+        >>> e3nn.norm(x, per_irrep=False)
+        1x0e [16.882]
     """
     jnp = _infer_backend(array.array)
 
     def f(x):
-        x = jnp.sum(x**2, axis=-1, keepdims=True)
+        if x is None:
+            return None
+
+        x = jnp.sum(jnp.conj(x) * x, axis=-1, keepdims=True)
         if not squared:
-            x = jnp.sqrt(x)
+            x_safe = jnp.where(x == 0.0, 1.0, x)
+            x_safe = jnp.sqrt(x_safe)
+            x = jnp.where(x == 0.0, 0.0, x_safe)
         return x
 
-    return IrrepsArray.from_list(
-        [(mul, "0e") for mul, _ in array.irreps],
-        [f(x) for x in array.list],
-        array.shape[:-1],
-        array.dtype,
-    )
+    if per_irrep:
+        return IrrepsArray.from_list(
+            [(mul, "0e") for mul, _ in array.irreps],
+            [f(x) for x in array.list],
+            array.shape[:-1],
+            array.dtype,
+        )
+    else:
+        return IrrepsArray("0e", f(array.array))
+
+
+def dot(a: IrrepsArray, b: IrrepsArray, per_irrep: bool = False) -> IrrepsArray:
+    """Dot product of two IrrepsArray.
+
+    Args:
+        a (IrrepsArray): first array (this array get complex conjugated)
+        b (IrrepsArray): second array
+        per_irrep (bool): if True, return the dot product of each irrep individually
+
+    Returns:
+        IrrepsArray: dot product of the two input arrays, as a scalar
+
+    Examples:
+        >>> x = e3nn.IrrepsArray("0e + 1e", jnp.array([1.0j, 1.0, 0.0, 0.0]))
+        >>> y = e3nn.IrrepsArray("0e + 1e", jnp.array([1.0, 2.0, 1.0, 1.0]))
+        >>> e3nn.dot(x, y)
+        1x0e [2.-1.j]
+
+        >>> e3nn.dot(x, y, per_irrep=True)
+        1x0e+1x0e [0.-1.j 2.+0.j]
+    """
+    jnp = _infer_backend([a.array, b.array])
+
+    a = a.simplify()
+    b = b.simplify()
+
+    if a.irreps != b.irreps:
+        raise ValueError("Dot product is only defined for IrrepsArray with the same irreps.")
+
+    if per_irrep:
+        out = []
+        dtype = a.dtype
+        for x, y in zip(a.list, b.list):
+            if x is None or y is None:
+                out.append(None)
+            else:
+                out.append(jnp.sum(jnp.conj(x) * y, axis=-1, keepdims=True))
+                dtype = out[-1].dtype
+        return IrrepsArray.from_list(
+            [(mul, "0e") for mul, _ in a.irreps],
+            out,
+            a.shape[:-1],
+            dtype,
+        )
+    else:
+        out = 0.0
+        for x, y in zip(a.list, b.list):
+            if x is None or y is None:
+                continue
+            out = out + jnp.sum(jnp.conj(x) * y, axis=(-2, -1))
+        if isinstance(out, float):
+            shape = jnp.broadcast_shapes(a.shape[:-1], b.shape[:-1])
+            return IrrepsArray.zeros("0e", shape, dtype=a.dtype)
+        return IrrepsArray("0e", out[..., None])
 
 
 def normal(
@@ -1326,7 +1401,7 @@ class _IndexUpdateRef:
                 list=[fn(x, y, mul, ir) for (mul, ir), x, y in zip(self.irreps, self.list, values.list)],
             )
 
-        raise NotImplementedError(f"x.add[i].set(v) with v={type(values)} is not implemented.")
+        raise NotImplementedError(f"x.at[i].set(v) with v={type(values)} is not implemented.")
 
     def add(self, values: Any) -> IrrepsArray:
         index = self.index
@@ -1377,7 +1452,7 @@ class _IndexUpdateRef:
                 list=[fn(x, y, mul, ir) for (mul, ir), x, y in zip(self.irreps, self.list, values.list)],
             )
 
-        raise NotImplementedError(f"x.add[i].add(v) with v={type(values)} is not implemented.")
+        raise NotImplementedError(f"x.at[i].add(v) with v={type(values)} is not implemented.")
 
 
 class _MulIndexSliceHelper:
