@@ -128,11 +128,14 @@ class SphericalSignal:
         return SphericalSignal(jnp.zeros((res_beta, res_alpha), dtype), quadrature, p_val=p_val, p_arg=p_arg)
 
     def __repr__(self) -> str:
-        return (
-            "SphericalSignal("
-            f"res_beta={self.res_beta}, res_alpha={self.res_alpha}, "
-            f"quadrature={self.quadrature}, p_val={self.p_val}, p_arg={self.p_arg})"
-        )
+        if self.ndim >= 2:
+            return (
+                "SphericalSignal("
+                f"res_beta={self.res_beta}, res_alpha={self.res_alpha}, "
+                f"quadrature={self.quadrature}, p_val={self.p_val}, p_arg={self.p_arg})"
+            )
+        else:
+            return f"SphericalSignal({self.grid_values})"
 
     def __mul__(self, scalar: float) -> "SphericalSignal":
         """Multiply SphericalSignal by a scalar."""
@@ -488,13 +491,13 @@ jax.tree_util.register_pytree_node(
 )
 
 
-def s2_dirac(position: jnp.ndarray, lmax: int, *, p_val: int, p_arg: int) -> e3nn.IrrepsArray:
+def s2_dirac(position: Union[jnp.ndarray, e3nn.IrrepsArray], lmax: int, *, p_val: int, p_arg: int) -> e3nn.IrrepsArray:
     r"""Spherical harmonics expansion of a Dirac delta on the sphere.
 
     The integral of the Dirac delta is 1.
 
     Args:
-        position (`jax.numpy.ndarray`): position of the delta, shape ``(3,)``.
+        position (`jax.numpy.ndarray` or `IrrepsArray`): position of the delta, shape ``(3,)``.
             It will be normalized to have a norm of 1.
 
         lmax (int): maximum degree of the spherical harmonics expansion
