@@ -1,9 +1,9 @@
 from e3nn_jax._src.linear_haiku import Linear
 from e3nn_jax._src.mlp_haiku import MultiLayerPerceptron
 from e3nn_jax._src.fc_tp_haiku import FullyConnectedTensorProduct
-from e3nn_jax._src.batchnorm import BatchNorm
-from e3nn_jax._src.dropout import Dropout
-from e3nn_jax._src.symmetric_tensor_product import SymmetricTensorProduct
+from e3nn_jax._src.batchnorm_haiku import BatchNorm
+from e3nn_jax._src.dropout_haiku import Dropout
+from e3nn_jax._src.symmetric_tensor_product_haiku import SymmetricTensorProduct
 
 
 __all__ = [
